import typer
import requests
from rich import print

import RPi.GPIO as GPIO
from mfrc522 import SimpleMFRC522
import spotipy
from spotipy.oauth2 import SpotifyClientCredentials, SpotifyOAuth

app = typer.Typer()

test_uri = "spotify:track:3v9GhZEIMWPJcWH0tbbSyT"  # Cancre - Vol de nuit
test_device = "Euterpe"


def get_account():
    r = requests.get("http://localhost:8000/accounts")
    accounts = r.json()
    return accounts[0]


@app.command()
def play():
    print("[blue]Play demo ...[/blue]")
    account = get_account()
    scope = "user-read-playback-state,user-modify-playback-state"
    sp = spotipy.Spotify(
        auth_manager=SpotifyOAuth(
            client_id=account["client_id"],
            client_secret=account["client_secret"],
            redirect_uri="http://localhost:8000/callback",
            scope=scope,
            open_browser=False,
        )
    )
    device = [d for d in sp.devices()["devices"] if d["name"] == test_device][0]
    print(f"[blue]Device:[/blue] {device['name']}")
    track = sp.track(test_uri)
    print(f"[blue]Currently playing:[/blue] {track['name']} by {track['artists'][0]['name']}")
    # Change track
    sp.start_playback(uris=[test_uri], device_id=device["id"])


@app.command()
def run():
    print("[blue]Starting server...[/blue]")
    reader = SimpleMFRC522()
    scope = "user-read-playback-state,user-modify-playback-state"
    sp = spotipy.Spotify(
        auth_manager=SpotifyOAuth(
            client_id=account["client_id"],
            client_secret=account["client_secret"],
            redirect_uri="http://localhost:8000/callback",
            scope=scope,
            open_browser=False,
        )
    )
    device = [d for d in sp.devices()["devices"] if d["name"] == test_device][0]

    while True:
        try:
            id, text = reader.read()
            print(f"Reading card {id}...")
            print(f"Search corresponding peering ...")
            r = requests.get(f"http://localhost:8000/peerings/{id}")
            peering = r.json()
            print(f"Found peering: {peering}")
            print(f"Play track {peering['spotify_id']} ...")
            track = sp.track(test_uri)
            sp.start_playback(uris=[test_uri], device_id=device["id"])
        finally:
            GPIO.cleanup()


@app.command()
def read():
    reader = SimpleMFRC522()

    try:
        id, text = reader.read()
        print(id)
        print(text)
    finally:
        GPIO.cleanup()


if __name__ == "__main__":
    app()



from pydantic import BaseModel


class SpotifyAccountBase(BaseModel):
    client_id: str
    client_secret: str


class SpotifyAccountCreate(SpotifyAccountBase):
    pass


class SpotifyAccount(SpotifyAccountBase):
    id: int

    class Config:
        from_attributes = True


class PeeringBase(BaseModel):
    nfc_id: int
    spotify_id: int | None = None


class PeeringCreate(PeeringBase):
    pass


class Peering(PeeringBase):
    id: int

    class Config:
        from_attributes = True



from sqlalchemy.orm import Session

from . import models, schemas


def get_spotify_account(db: Session, spotify_id: str):
    return db.query(models.SpotifyAccount).filter(models.SpotifyAccount.id == spotify_id).first()


def get_spotify_accounts(db: Session, skip: int = 0, limit: int = 100):
    return db.query(models.SpotifyAccount).offset(skip).limit(limit).all()


def create_spotify_account(db: Session, account: schemas.SpotifyAccountCreate):
    db_account = models.SpotifyAccount(**account.dict())
    db.add(db_account)
    db.commit()
    db.refresh(db_account)
    return db_account


def get_peering(db: Session, peering_id: int):
    return db.query(models.Peering).filter(models.Peering.id == peering_id).first()


def get_peerings(db: Session, skip: int = 0, limit: int = 100):
    return db.query(models.Peering).offset(skip).limit(limit).all()


def create_peering(db: Session, peering: schemas.PeeringCreate):
    db_peering = models.Peering(**peering.dict())
    db.add(db_peering)
    db.commit()
    db.refresh(db_peering)
    return db_peering



from fastapi import Depends, FastAPI
from sqlalchemy.orm import Session

from .core import crud, schemas
from .core import models
from .core.database import SessionLocal, engine

models.Base.metadata.create_all(bind=engine)

app = FastAPI()


# Dependency
def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


@app.get("/")
async def root():
    return {"message": "Welcome to euterpe"}


@app.post("/peering/", response_model=schemas.Peering)
def create_peering(peering: schemas.PeeringCreate, db: Session = Depends(get_db)):
    # db_peering = crud.get_peering_by_email(db, email=peering.email)
    # if db_peering:
    #     raise HTTPException(status_code=400, detail="Email already registered")
    return crud.create_peering(db=db, peering=peering)


@app.get("/peerings/", response_model=list[schemas.Peering])
def read_peerings(skip: int = 0, limit: int = 100, db: Session = Depends(get_db)):
    peerings = crud.get_peerings(db, skip=skip, limit=limit)
    return peerings


@app.post("/account/", response_model=schemas.SpotifyAccount)
def create_account(account: schemas.SpotifyAccountCreate, db: Session = Depends(get_db)):
    return crud.create_spotify_account(db=db, account=account)


@app.get("/accounts/", response_model=list[schemas.SpotifyAccount])
def read_accounts(skip: int = 0, limit: int = 100, db: Session = Depends(get_db)):
    return crud.get_spotify_accounts(db, skip=skip, limit=limit)



import typer
from commands import accounts
from commands import server

app = typer.Typer()
app.add_typer(accounts.app, name="accounts")
app.add_typer(server.app, name="server")


if __name__ == "__main__":
    app()



from sqlalchemy import Boolean, Column, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

from .database import Base


class SpotifyAccount(Base):
    __tablename__ = "spotify_accounts"

    id = Column(Integer, primary_key=True, index=True)
    client_id = Column(String, index=True)
    client_secret = Column(String, index=True)


class Peering(Base):
    __tablename__ = "peering"

    id = Column(Integer, primary_key=True, index=True)
    nfc_id = Column(Integer, index=True)
    song_id = Column(Integer, index=True)



import typer
import requests
from rich import print

app = typer.Typer()


@app.command()
def create(client_id: str, client_secret):
    body = {"client_id": client_id, "client_secret": client_secret}
    r = requests.post("http://localhost:8000/account/", json=body)
    if r.status_code == 200:
        print("[green]Account created[/green]")
    else:
        print(f"[yellow]Error while creating account {client_id}:[/yellow] {r.reason}")


@app.command()
def list():
    r = requests.get("http://localhost:8000/accounts")
    accounts = r.json()
    print("[blue]Accounts:[/blue]")
    for account in accounts:
        print(f"- {account['client_id']}")


if __name__ == "__main__":
    app()
